package algorithms;

import types.procedures.ICombine;
import types.procedures.IConvert;
import types.procedures.ISelect;
import types.structures.IList;

// Represents list-processing algorithms.
public class ListAlgorithms {
	
	// Produces all elements in list satisfying predicate, in order.
	public static IList filter( IList list, ISelect predicate ) {
		if ( list.isEmpty() ) {
			return list;
		} else {
			Object first = list.getFirst();
			IList recur = filter( list.getRest(), predicate );
			
			if ( predicate.select(first) ) {
				return recur.cons( first );
			} else {
				return recur;
			}
		}
	}
	
	// Produces all elements in list, in order, transformed by mapping.
	public static IList map( IList list, IConvert mapping ) {
		if ( list.isEmpty() ) {
			return list;
		} else {
			Object first = list.getFirst();
			IList recur = map( list.getRest(), mapping );

			return recur.cons( mapping.convert(first) );
		}
	}
	
	// Tells whether any list elements satisfy predicate.
	public static boolean orMap( IList list, ISelect predicate ) {
		while( !list.isEmpty() ) {
			if( predicate.select(list.getFirst()) ) {
				return true;
			} else {
				list = list.getRest();
			}
		}
		return false;
	}
	
	// Tells whether all list elements satisfy predicate.
	public static boolean andMap( IList list, ISelect predicate ) {
		while( !list.isEmpty() ) {
			if( !predicate.select(list.getFirst()) ) {
				return false;
			} else {
				list = list.getRest();
			}
		}
		return true;
	}
	
	// Produces the combination (Left->right) of all list elements with the init value.
	public static Object foldLeft( IList list, Object init, ICombine join ) {
		if ( list.isEmpty() ) {
			return init;
		} else {
			return foldLeft( list.getRest(), join.combine( list.getFirst(), init ), join );
		}
	}
	
	// Produces the combination (Right->left) of all list elements with the init value.
	public static Object foldRight( IList list, Object init, ICombine join ) {
		if( list.isEmpty() ) {
			return init;
		} else {
			return join.combine( list.getFirst(), foldRight( list.getRest(), init, join ) );
		}
	}

}
