package structures.set;

import exceptions.NotFoundException;
import algorithms.TreeAlgorithms;
import procedures.combinations.ListCons;
import procedures.predicates.AVLInvariant;
import structures.list.Empty;
import structures.tree.Leaf;
import types.procedures.ICompare;
import types.procedures.ISelect;
import types.structures.IList;
import types.structures.ISame;
import types.structures.ISet;
import types.structures.ITree;

// Represents a set with ordered elements.
public class OrderedSet implements ISet {

	private static ISelect balanceInvariant = new AVLInvariant();

	public ICompare order;
	private ITree tree;
	
	public OrderedSet( ICompare order ) {
		this.order = order;
		this.tree = new Leaf();
	}
	
	private OrderedSet( ICompare order, ITree tree ) {
		this.order = order;
		this.tree = tree;
	}
	
	// Tells whether this contains no elements.
	public boolean isEmpty() {
		return this.tree.isLeaf();
	}

	// Tells whether this contains element.
	public boolean member(ISame element) {
		try {
			TreeAlgorithms.find( this.tree, element, this.order );
			return true;
		}
		catch( NotFoundException e ) {
			return false;
		}
	}

	// Produces a copy of this containing element.
	public ISet insert(ISame element) {
		ITree tree =
			TreeAlgorithms.insertBalanced(
				this.tree, element, this.order, balanceInvariant );
		return new OrderedSet( this.order, tree );
	}

	// Produces all elements in this sorted by the given ordering.
	public IList elements() {
		return (IList)TreeAlgorithms.foldRight( this.tree, new Empty(), new ListCons() );
	}
	
	// Produces a string displaying this.
	public String toString() {
		return "(set " + this.elements().toString() + ")";
	}	

}
