package structures.tree;

import types.structures.ISame;
import types.structures.ITree;

// Represents a node in a binary tree
public class Node extends ATree {
	
	private int size;
	public Object data;
	public ATree left;
	public ATree right;

	public Node( ATree left, Object obj, ATree right ) {
		this.size = 1 + Math.max(left.getHeight(), right.getHeight());
		this.data = obj;
		this.left = left;
		this.right = right;
	}

	// Tells whether this is the same as that.
	public boolean same( ISame that ) {
		if ( that instanceof ITree ) {
			ITree tree = (ITree)that;
			return
				!tree.isLeaf() &&
				((ISame)this.data).same((ISame)tree.getData()) &&
				this.left.same(tree.getLeft()) &&
				this.right.same(tree.getRight());
		} else {
			return false;
		}
	}
	
	// Produces the height of this (distance to a leaf)
	public int getHeight() {
		return size;
	}

	// Tells that this is not a leaf
	public boolean isLeaf() {
		return false;
	}

	// Produces data stored in this
	public Object getData() {
		return this.data;
	}

	// Produces the left child of this
	public ITree getLeft() {
		return this.left;
	}

	// Produces the right child of this
	public ITree getRight() {
		return this.right;
	}
	
	// Produces a string to display this.
	public String toString() {
		return 
			"(node " + this.left.toString() + " " +
			this.data.toString() + " " + this.right.toString() + ")";
	}

}
